import yt_dlp
import tkinter as tk
from tkinter import messagebox, scrolledtext
from tkinter import ttk

class YouTubeVideoDownloader:
    def __init__(self, root):
        self.root = root
        self.root.title("YouTube Video Downloader")
        self.root.geometry("600x350")
        self.root.configure(bg="#f0f0f0")

        # Title
        self.title_label = tk.Label(root, text="YouTube Video Downloader", font=("Arial", 16), bg="#f0f0f0")
        self.title_label.pack(pady=10)

        # Frame for input
        input_frame = tk.Frame(root, bg="#f0f0f0")
        input_frame.pack(pady=10)

        self.url_label = tk.Label(input_frame, text="Enter YouTube Video URL:", bg="#f0f0f0")
        self.url_label.pack(side=tk.LEFT)

        self.url_entry = tk.Entry(input_frame, width=40)
        self.url_entry.pack(side=tk.LEFT, padx=5)

        self.download_button = tk.Button(input_frame, text="Download Video", command=self.download_video, bg="#4CAF50", fg="white")
        self.download_button.pack(side=tk.LEFT)

        # Status Display
        self.status_box = scrolledtext.ScrolledText(root, width=70, height=10, bg="#ffffff", fg="#333333", wrap=tk.WORD)
        self.status_box.pack(pady=10)

        # Progress Bar
        self.progress = ttk.Progressbar(root, orient="horizontal", length=500, mode="determinate")
        self.progress.pack(pady=10)

        self.ydl_opts = {
            'progress_hooks': [self.progress_hook]
        }

    def progress_hook(self, d):
        if d['status'] == 'downloading':
            self.progress['value'] = d['downloaded_bytes'] / d['total_bytes'] * 100
            self.status_box.delete(1.0, tk.END)
            self.status_box.insert(tk.END, f"Downloading: {d['downloaded_bytes'] / (1024 * 1024):.2f} MB of {d['total_bytes'] / (1024 * 1024):.2f} MB\n")
            self.root.update_idletasks()  # Update the UI

        elif d['status'] == 'finished':
            self.progress['value'] = 100
            self.status_box.insert(tk.END, "Download finished!\n")
            self.root.update_idletasks()  # Update the UI

    def download_video(self):
        video_url = self.url_entry.get()
        self.status_box.delete(1.0, tk.END)  # Clear previous messages
        self.progress['value'] = 0  # Reset progress bar

        if video_url:
            try:
                with yt_dlp.YoutubeDL(self.ydl_opts) as ydl:
                    ydl.download([video_url])
            except Exception as e:
                self.status_box.insert(tk.END, "Error: " + str(e) + "\n")
        else:
            self.status_box.insert(tk.END, "Please enter a valid YouTube video URL\n")

        self.url_entry.delete(0, tk.END)

    def run(self):
        self.root.mainloop()

if __name__ == "__main__":
    root = tk.Tk()
    app = YouTubeVideoDownloader(root)
    app.run()